@@ -10,6 +10,8 @@
 # v0.0.1 : 2021/12/31 release
 # v0.0.2 : 2022/02/07 format revised
 # v0.1.0 : 2022/06/01 tune to fit v3
+# v1.0.0 : 2022/06/08 change get Uart method
+# v1.0.1 : 2022/06/09 added headerShowAll()
  
 
 import serial
@@ -28,12 +30,22 @@ class header:
 	numTLVs = 0
 	subFrameNumber = 0
 
+class byteInfo:
+	headerByte = 32
+
 
 class MRR:
     
 	magicWord =  [b'\x01',b'\x02',b'\x03',b'\x04',b'\x05',b'\x06',b'\x07',b'\x08',b'\0x99']
+	magicLen = 8
 	port = ''
 	hdr = header
+	hdr0 = header()
+	hdr1 = header()
+	info = byteInfo
+	hdrA = [hdr0,hdr1]
+	headerDic = {0:hdr0, 1:hdr1}
+	
 	clearQBuf = True
 	# provide csv file dataframe
 	# real-time 
@@ -56,6 +68,8 @@ class MRR:
 	v3simo = []
 	v4simo = []
 	
+	buf = b''
+	
 	v1 = ([])
 	v2 = ([])
 	v3 = ([])
@@ -132,7 +146,21 @@ class MRR:
 		print("***End Of Header***") 
 		
 	
-	def list2df(self,l1,l2,l3,l4):
+	def headerShowAll(self):
+		for i in range(2):
+			print(f'hdrA({i})  : fn = { self.headerDic[i].frameNumber} ')
+			print("***Header***********")  # 32 bytes
+			print("Version:     \t%x "%( self.headerDic[i].version))
+			print("TotalPackLen:\t%d "%( self.headerDic[i].totalPackLen))
+			print("Platform:    \t%X "%( self.headerDic[i].platform))
+			print("PID(frame#): \t%d "%( self.headerDic[i].frameNumber))
+			print("Inter-frame Processing Time:\t{:d} us".format( self.headerDic[i].cpuProcessTime))
+			print("number of Objects:     \t%d "%( self.headerDic[i].numOBJs))
+			print("numTLVs:     \t%d "%( self.headerDic[i].numTLVs))
+			print("subframe#  : \t%d "%( self.headerDic[i].subFrameNumber))
+			print("***End Of Header***") 
+	
+	def list2df(self,chk,l1,l2,l3,l4):
 		#print("---------------list2df: v1----------------")
 		#print(l1)
 		
@@ -142,8 +170,108 @@ class MRR:
 		ll4 = pd.DataFrame(l4,columns=self.v4_col_names_rt)
 		#print("------ll1---------list2df: v1----------------")
 		#print(ll1)
-		return (ll1,ll2,ll3,ll4)
+		return (chk,ll1,ll2,ll3,ll4)
+		
+	 
+	#################################################################################
+	# FUNCTION: jb_getUartBuf(); get from serial port into buffer uartBuf[]
+	#################################################################################
+	JB_syncFlag = 0
+	def jb_getUartBuf(self,disp = None):
+		idx = 0
+		self.buf = b''
+		self.buf1 = b''
+		bufDic = {}
+		version = 0 
+		totalPackLen = 0 
+		platform = 0
+		frameNumber = 0 
+		cpuProcessTime = 0
+		numOBJs = 0 
+		numTLVs = 0
+		subFrameNumber = 0
+		hdr0 = header()
+		hdr1 = header()
+		self.headerDic = {0:hdr0, 1:hdr1}
+		
+		while True:
+			if self.JB_syncFlag == 0:
+				ch = self.port.read() # here ch type is byte
+				if ch == self.magicWord[idx]: # 0..7
+					self.buf += ch
+					idx += 1 # 1..8
+					if idx == 8:
+						idx = 8
+						self.JB_syncFlag = 1
+						self.JB_t0 = time.time()
+						self.JB_tMagic = time.time()
+			else:
+				self.JB_syncFlag = 0 # magic word ch mismatch loop back again
+				idx = 0 # init
+				self.buf = b""
+				
+			if self.JB_syncFlag == 1:
+				# read next 32 bytes for header version, len, platform, fn
+				for i in range(self.info.headerByte):
+					ch = self.port.read()
+					self.buf += ch
+				if len(self.buf) == (self.magicLen + self.info.headerByte): 
+					try: 
+						(version,totalPackLen,platform,frameNumber,cpuProcessTime,numOBJs,numTLVs,subFrameNumber
+						) = struct.unpack('8I', self.buf[8:40])
+						
+						(self.hdr.version,self.hdr.totalPackLen,self.hdr.platform,self.hdr.frameNumber,
+						self.hdr.cpuProcessTime,self.hdr.numOBJs,self.hdr.numTLVs,self.hdr.subFrameNumber
+						) = (version,totalPackLen,platform,frameNumber,cpuProcessTime,numOBJs,numTLVs,subFrameNumber)
+						#if self.dbg == True:
+						if disp:
+							self.headerShow()
+						 
+					except:
+						#if self.dbg == True:
+						print("(Header)Improper TLV structure found: ")
+						return (bufDic,self.headerDic)
+					
+					fn = self.hdr.frameNumber
+					self.JB_syncFlag = 2
+					
+				else:
+					self.JB_syncFlag = 0 # len mismatch loop back again
+					idx = 0
+					self.buf = b""
 
+			if self.JB_syncFlag == 2:
+				
+				dataLen =  totalPackLen - self.magicLen - self.info.headerByte
+				#print(f'----------JB_sync=2  data length: {dataLen}------------')
+				r = self.port.read(dataLen) # read rest of bytes
+				self.buf += r
+				#self.port.flushInput() 
+				dt = time.time()- self.JB_t0
+				self.JB_tBuf = time.time()
+				
+				if self.hdr.subFrameNumber == 1:
+					bufDic[1]    = self.buf
+					(hdr1.version,hdr1.totalPackLen,hdr1.platform,hdr1.frameNumber,hdr1.cpuProcessTime,hdr1.numOBJs,hdr1.numTLVs,hdr1.subFrameNumber
+					) = (version,totalPackLen,platform,frameNumber,cpuProcessTime,numOBJs,numTLVs,subFrameNumber)
+					#headerDic[1] = hdr1
+					self.JB_syncFlag == 0
+					#print(f"--------1-----------{hdr1.totalPackLen}--------{ time.time() - self.JB_t0 }")
+					self.port.flushInput() 
+					return (bufDic,self.headerDic)
+				 
+				if self.hdr.subFrameNumber == 0:
+					bufDic[0] = self.buf
+					
+					(hdr0.version,hdr0.totalPackLen,hdr0.platform,hdr0.frameNumber,hdr0.cpuProcessTime,hdr0.numOBJs,hdr0.numTLVs,hdr0.subFrameNumber
+					) =  (version,totalPackLen,platform,frameNumber,cpuProcessTime,numOBJs,numTLVs,subFrameNumber)
+					#headerDic[0] = hdr0
+					self.JB_syncFlag == 0
+					idx = 0
+					self.buf = b""
+					#print(f"--------0-----------{hdr0.totalPackLen}--------{ time.time() - self.JB_t0 }")
+				 
+	#################################################################
 #
 # TLV: Type-Length-Value
 # read TLV data
@@ -151,20 +279,21 @@ class MRR:
 #     disp: True:print message
 #			False: hide printing message
 # output:(return parameter)
-# (chk, v1, v2, v3, v4)
-#  chk:return data status 
-#   0: empty: tlv = 0
-#   1: data output
-#  
-#   10: idle
-#   99: error
-# 
+# (pass_fail, v1, v2, v3, v4)
+#  pass_fail: True: Data available    False: Data not available
 #
 #  v1: Detected Object List infomation
 #  v2: Cluster Output information
 #  v3: Tracking Output information
 #  v4: Parking Assist Infomation
 #
+#  chk:  
+#   0: empty: tlv = 0
+#   1: data output
+#  
+#   10: idle
+#   99: error
+#
 #
 	def tlvRead(self,disp,df = None):
 		#print("---tlvRead---")
@@ -179,199 +308,110 @@ class MRR:
 		pbyte = 16
 		
 		typeCnt = 0 # for check with tlvCount, if typeCnt equal to tlvCcount the process will completed 
+		idxFrom = 0
+		idxTo = 0
+		magicLen  = 8
+		headerLen = 32
+		typeLen   = 8
+		objUnitLen = 4
 		
+		''' 
+		(uartBuf,hdrA) = self.jb_getUartBuf()
+		print(f'uartBuf[0]:\n {uartBuf[0]}\n--------------------\nuartBuf[1]:\n {uartBuf[0]}\n')
+		'''
+		
+		hdr_sf = 0 
 		while True:
-			try:
-				ch = self.port.read()
-			except:
-				#return (False,v6,v7,v8,v9)
-				return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
-			#print(str(ch))
+			
 			if lstate == 'idle':
-				#print(self.magicWord)
-				if ch == self.magicWord[idx]:
-					#print("*** magicWord:"+ "{:02x}".format(ord(ch)) + ":" + str(idx))
-					idx += 1
-					if idx == 8:
-						idx = 0
-						lstate = 'header'
-						rangeProfile = b""
-						sbuf = b""
+				if self.sm == True:
+					print(f'(SM) {lstate}') 
+					
+				(uartBuf,hdrA) = self.jb_getUartBuf(disp = disp)
+				lstate = 'block0'
+				idxFrom = 0
+				idxTo = 0
+				
+			if lstate == 'block0': #subFrameNumber = 0 
+				hdr_sf = 0
+				if self.sm == True:
+						print(f'(SM) {lstate}') 
+						
+				if hdrA[0].numOBJs == 0 and hdrA[0].numTLVs == 0 and hdrA[0].subFrameNumber == 0:
+					self.v3 = []; self.v3df = []
+					lstate = 'block1'
 				else:
-					#print("not: magicWord state:")
-					idx = 0
-					rangeProfile = b""
+					try:
+						sbuf = uartBuf[0][40:]
+					except:
+						return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
+					#print("".join("{:02x}".format(c) for c in sbuf)) 
+					idxFrom = 0
+					idxTo = typeLen + objUnitLen
+					tlvCount = hdrA[0].numTLVs
+					lstate = 'Units'
+			
+			if lstate ==  'block1':
+				hdr_sf = 1
+				sbuf = b''
+				sbuf = uartBuf[1][40:] #remove header
+				if self.sm == True:
+						print(f'(SM) {lstate}') 
+						
+				'''
+				print(f'uartBuf-header Length= {len(sbuf)} objs:{hdrA[1].numOBJs} tlv#:{hdrA[1].numTLVs} subFrame:{hdrA[1].subFrameNumber}')
+				print("".join("{:02x}".format(c) for c in sbuf))  
+				'''
+				if hdrA[1].numOBJs == 0 and hdrA[1].numTLVs == 0 and hdrA[1].subFrameNumber == 1:
+					self.v1 = [];self.v1df = []; self.v2 = [];self.v2df = [];self.v4 = [];self.v4df = []
 					return self.list2df(10,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (10,self.v1,self.v2,self.v3,self.v4)
+				idxFrom = 0
+				idxTo = typeLen + objUnitLen
+				tlvCount = hdrA[1].numTLVs
+				lstate = 'Units'
 		
-			elif lstate == 'header':
-				sbuf += ch
-				idx += 1
-				if idx == 32:   
-					# print(":".join("{:02x}".format(c) for c in sbuf))  
-					# [header - Magicword]
-					try: 
-						(self.hdr.version,self.hdr.totalPackLen,self.hdr.platform,self.hdr.frameNumber,self.hdr.cpuProcessTime,self.hdr.numOBJs,self.hdr.numTLVs,self.hdr.subFrameNumber
-						) = struct.unpack('8I', sbuf)
-						if self.dbg == True:
-							print("\n================= header({:}) ===========numTLVs:{:}".format(self.hdr.frameNumber,self.hdr.numTLVs))
-					except:
-						if self.dbg == True:
-							print("(Header)Improper TLV structure found: ")
-						return self.list2df(99,self.v1,self.v2,self.v3,self.v4) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
-					
-					if disp == True:  
-						self.headerShow()
-					
-					tlvCount = self.hdr.numTLVs
-					sbuf = b""
-					idx = 0
-					lstate = 'TL'
-					typeCnt = 0
-					self.clearBuffer_v124()
-					if self.sm == True:
-						print("(Header)")
-					if self.hdr.numTLVs == 0:
-						lstate = 'idle'
-						return self.list2df(0,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (0,self.v1,self.v2,self.v3,self.v4)
-						
-					
-
-				elif idx > 48:
-					idx = 0
-					lstate = 'idle'
-					return self.list2df(10,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (10,self.v1,self.v2,self.v3,self.v4)
-					
-			elif lstate == 'TL': #TLV Header type/length
-				sbuf += ch
-				idx += 1
-				if idx == 8:
-					#print(":".join("{:02x}".format(c) for c in sbuf))
-					try: 
-						ttype,self.tlvLength = struct.unpack('2I', sbuf)
-						#print("(TL)numTLVs({:d}): tlvCount({:d})-------ttype:tlvLength:v{:d}:{:d}".format(self.hdr.numTLVs,tlvCount,ttype,self.tlvLength))
-						if ttype not in typeList or self.tlvLength > self.hdr.totalPackLen:
-							if self.dbg == True:
-								print("(TL)Improper TL Length(hex):(T){:d} (L){:x} numTLVs:{:d}".format(ttype,self.tlvLength,self.hdr.numTLVs))
-							sbuf = b""
-							idx = 0
-							lstate = 'idle'
-							self.port.flushInput()
-							return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (s99elf.v1,self.v2,self.v3,self.v4)
-					except:
-						 
-						if self.dbg == True:
-							print("TL unpack Improper Data Found:")
-						self.port.flushInput()
-						return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
-					
-					lstate = typeString[ttype] 
-					#print("(TL)lstate:{:}".format(lstate))
-					
-					if self.sm == True:
-						print("(TL:{:d})=>({:})".format(tlvCount,lstate))
-						
-					tlvCount -= 1
-					idx = 0  
-					sbuf = b""
-					
-			elif lstate == 'V1' or lstate == 'V2' or lstate == 'V3': 
-				sbuf += ch
-				idx += 1
-				if (idx%self.tlvLength == 0):
-					typeCnt += 1
-					try:
-						if self.dbg == True:
-							print("========== {} =============:tlvLength={:}  idx={:}".format(msgString[lstate],self.tlvLength,idx))
-						#print(":".join("{:02x}".format(c) for c in sbuf))
-						objs, xyzQFormat = struct.unpack('2H', sbuf[0:4])
-						vUnit = 1 / 2**xyzQFormat
-						if self.dbg == True:
-							print("objs= {} xzQFormat= {} vUnit= {:.4f}".format(objs,xyzQFormat,vUnit))
-							#print(":".join("{:02x}".format(c) for c in sbuf[4:]))
-							
-						self.typeFunc(typeT = lstate,vUnit= vUnit ,objs=objs,ldata = sbuf[4:], df = df)
-						
-						if typeCnt != self.hdr.numTLVs:
-							if self.dbg == True:
-								print("({:}) -> (TL): typeCnt:{:} subFrame:{:}".format(lstate,typeCnt,self.hdr.subFrameNumber))
-							lstate = 'TL'
-						else:
-							if self.sm == True:
-								print(f"type Cnt: {typeCnt}   {lstate}")
-								print("({:}) -> (idle): typeCnt:{:}".format(lstate,typeCnt))
-							#lstate = 'idle'
-						sbuf = b""
-						idx = 0
-						
-					except:
-						if self.dbg == True:
-							print("({:})Improper Type {:} Value structure found: ".format(lstate,lstate))
-						return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
-						
-					if tlvCount == 0:
-						#print(f"==================== tlvCnt == 0  lstate = {lstate}    ===================")
-						lstate = 'idle'
-						if self.hdr.subFrameNumber == 1:
-							return self.list2df(1,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (1,self.v1,self.v2,self.v3,self.v4)
-						
-						
-			elif lstate == 'V4': # parking assist  
-				sbuf += ch
-				idx += 1
-				if (idx%self.tlvLength == 0):
-					typeCnt += 1
-					try:
-						if self.dbg == True:
-							print("========== {} ==============:lengthCnt={:}  idx={:}".format(msgString[lstate],self.tlvLength,idx))
-						#print(":".join("{:02x}".format(c) for c in sbuf))
-						objs, xyzQFormat = struct.unpack('2H', sbuf[0:4])
-						vUnit = 1 / 2**xyzQFormat
-						if self.dbg == True:
-							print("(V4)objs= {} xzQFormat= {} v4Unit= {:.4f} typeCnt:{:}".format(objs,xyzQFormat,vUnit,typeCnt))
-						ldata = sbuf[4:]
-						astA = [] 
-						for i in range(objs):
-							start = i * 2
-							end   = (i+1) * 2
-							ast, = struct.unpack('H', ldata[start:end]) 
-							astA.append(ast * vUnit)
-							
-						if self.dbg == True: 
-							print("V4 astA lenth:{:}".format(len(astA)))
-						
-						if (df == 'DataFrame'):
-							self.v4df.append((self.hdr.frameNumber,'v4',astA))
-						else:
-							self.v4.append((self.hdr.frameNumber,astA))
-						
-							#print("tid = ({:d}) ,posX:{:.4f} posY:{:.4f} posZ:{:.4f}".format(tid,posX,posY,posZ))
-						if typeCnt == self.hdr.numTLVs:
-							if self.dbg == True:
-								print("(V4) -> (idle): typeCnt:{:}".format(typeCnt))
-							lstate = 'idle'	
-							return self.list2df(1,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (1,self.v1,self.v2,self.v3,self.v4)
-						else:
-							if self.dbg == True:
-								print("(V4) -> (TL): typeCnt:{:}".format(typeCnt))
-							lstate = 'TL'
-						
-						sbuf = b""
-						idx = 0
-						 
-						 
-					except:
-						if self.dbg == True:
-							print("(V4)Improper Type 4 Value structure found: ")
-						
-						return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
-						
+			if lstate == 'Units':
+				#print(f"============= Units ================== from:to= [{idxFrom}:{idxTo}]")
+				#print("".join("{:02x}".format(c) for c in sbuf[idxFrom:idxTo])) 
+				ttype,self.tlvLength, objs,xyzQFormat = struct.unpack('2I2H', sbuf[idxFrom:idxTo]) 
+				if ttype not in typeList:
+					return self.list2df(99,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (99,self.v1,self.v2,self.v3,self.v4)
 					
+				vUnit = 1 / 2**xyzQFormat
+				if disp:
+					print(f'type/length: v{ttype}/{self.tlvLength} objs:{objs} xyzQFormat:{xyzQFormat} unit:{vUnit}')
+				lstate = typeString[ttype]
+				
+				idxFrom = idxTo
+				idxTo = idxFrom + self.tlvLength - objUnitLen 
+				#print(f"============= Units -> {lstate} ====== from:to= [{idxFrom}:{idxTo}]")
+			
+			if lstate == 'V1' or lstate == 'V2' or lstate == 'V4' or lstate == 'V3':
+				if self.sm == True:
+						print(f'(SM) {lstate}') 
+				tlvCount -= 1
+				self.typeFunc(typeT = lstate ,vUnit= vUnit ,objs=objs,ldata = sbuf[idxFrom:idxTo], df = df)
+				idxFrom = idxTo
+				idxTo = idxFrom + typeLen + objUnitLen 
+				lstate = 'Units'
+			
+			if tlvCount == 0 and hdr_sf == 0:
+				lstate = 'block1'
+				idxTo = 0
+				idxFrom = 0
+				
+			if tlvCount == 0 and hdr_sf == 1:
+				lstate = 'idle'
+				return self.list2df(1,self.v1df,self.v2df,self.v3df,self.v4df) if (df == 'DataFrame') else (1,self.v1,self.v2,self.v3,self.v4)
+			
 			
 	def typeFunc(self,typeT = None ,vUnit= None,objs= None,ldata = None, df = None):
 		#print('typeFunc===> {}  vUnit:{:}'.format(typeT,vUnit))
 		#print(":".join("{:02x}".format(c) for c in ldata))
 		
 		if typeT == 'V1':
+			self.v1df = ([])
+			self.v1   = ([])
 			for i in range(objs):
 				start = i * 10
 				end   = (i+1) * 10
@@ -391,6 +431,8 @@ class MRR:
 					self.v1.append((self.hdr.frameNumber,d,p,x,y,z))
 				
 		if typeT == 'V2':
+			self.v2df = ([])
+			self.v2   = ([])
 			for i in range(objs):
 				start = i * 8
 				end   = (i+1) * 8
@@ -406,8 +448,9 @@ class MRR:
 				else:
 					self.v2.append((self.hdr.frameNumber,ccX,ccY,csX,csY)) 
 					
-		if typeT == 'V3':
-			self.clearBuffer_v3()
+		if typeT == 'V3': 
+			self.v3 = ([])
+			self.v3df = ([])
 			for i in range(objs):
 				start = i * 12
 				end   = (i+1) * 12
@@ -422,11 +465,26 @@ class MRR:
 					print("v3 => X:{:.4f} Y:{:.4f}  XD:{:.4f}  YD:{:.4f} Xsize:{:.4f} Ysize:{:.4f}".format(X,Y,XD,YD,Xsize,Ysize))
 				if (df == 'DataFrame' ):
 					self.v3df.append((self.hdr.frameNumber,'v3',X,Y,XD,YD,Xsize,Ysize))
-				else:
-					 
+				else: 
 					self.v3.append((self.hdr.frameNumber,X,Y,XD,YD,Xsize,Ysize))
 					
-
+		if typeT == 'V4':
+			self.v4df = ([])
+			self.v4   = ([])
+			for i in range(objs):
+				astA = [] 
+				for i in range(objs):
+					start = i * 2
+					end   = (i+1) * 2
+					ast, = struct.unpack('H', ldata[start:end]) 
+					astA.append(ast * vUnit)
+					
+			if (df == 'DataFrame'):
+				self.v4df.append((self.hdr.frameNumber,'v4',astA))
+			else:
+				self.v4.append((self.hdr.frameNumber,astA))
+	
+	
 
 	def getRecordData(self,frameNum):
 		s_fn = frameNum + self.sim_startFN
