@@ -11,6 +11,7 @@
 # ver:2.0.2 2022/05/12
 # ver:2.0.3 2022/05/12
 # ver:2.0.4 2022/05/15
+# ver:2.0.5 2023/02/10
 #===========================================
 # output: V6,V7,V8,v9 Raw data
 #
@@ -92,7 +93,7 @@ class TrafficMD_I480:
 		print("V6: Point Cloud Spherical")
 		print("V6 structure: [(range,azimuth,elevation,doppler,sx,sy,sz),......]")
 		print("V7: Target Object List")
-		print("V7 structure: [(tid,posX,posY,posZ,velX,velY,velZ,accX,accY,accZ),....]")
+		print("V7 structure: [(posX,posY,posZ,velX,velY,velZ,accX,accY,accZ,tid),....]") #revised text change "tid" position
 		print("V8: Target Index")
 		print("V8 structure: [id1,id2....]")
 		print("V9: Point Cloud Side Info")
